<?php
/*
Plugin Name: SearchWP Legacy Data Removal
Plugin URI: https://searchwp.com/
Description: Removes data from legacy versions of SearchWP
Version: 1.0.0
Author: SearchWP
Author URI: https://searchwp.com/

Copyright 2020 SearchWP

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>.
*/

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! defined( 'SEARCHWP_LDR_VERSION' ) ) {
	define( 'SEARCHWP_LDR_VERSION', '1.0.0' );
}

/**
 * Instantiate the updater.
 */
if ( ! class_exists( 'SearchWP_LDR_Updater' ) ) {
	include_once( dirname( __FILE__ ) . '/vendor/updater.php' );
}

function searchwp_ldr_update_check() {

	if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
		return false;
	}

	if ( ! defined( 'SEARCHWP_PREFIX' ) ) {
		return false;
	}

	if ( ! defined( 'SEARCHWP_EDD_STORE_URL' ) ) {
		return false;
	}

	if ( ! class_exists( '\\SearchWP\\License' ) ) {
		return false;
	}

	$searchwp_ldr_updater = new SearchWP_LDR_Updater( SEARCHWP_EDD_STORE_URL, __FILE__, array(
			'item_id' 	=> 219805,
			'version'   => SEARCHWP_LDR_VERSION,
			'license'   => \SearchWP\License::get_key(),
			'item_name' => 'Legacy Data Removal',
			'author'    => 'SearchWP',
			'url'       => site_url(),
		)
	);

	return $searchwp_ldr_updater;
}

add_action( 'admin_init', 'searchwp_ldr_update_check' );

class SearchWPLegacyDataRemoval {

	function __construct() {
		add_action( 'admin_init', function() {
			if (
				isset( $_GET['searchwp-action'] )
				&& 'remove-3x' === $_GET['searchwp-action']
				&& isset( $_GET['searchwp_ldr'] )
				&& wp_verify_nonce( $_GET['searchwp_ldr'], 'searchwp-remove-3x' )
			) {
				$this->remove_3x();
				add_action( 'admin_notices', function() {
					?>
					<div class="notice notice-success">
						<p>SearchWP 3.x legacy data has been removed!</p>
					</div>
					<?php
				} );
			}
		} );

		add_action( 'admin_menu', function() {
			add_management_page(
				'SearchWP Legacy Data Removal',
				'SearchWP Legacy Data Removal',
				'manage_options',
				'searchwp-legacy-data-removal',
				array( $this, 'view' )
			);
		} );
	}

	/**
	 * Tools screen view.
	 *
	 * @return void
	 */
	public function view() {
		?>
		<div class="wrap">
			<h1>SearchWP Legacy Data Removal</h1>
			<div id="searchwp-legacy-data-removal">

				<p>The following legacy versions of SearchWP have been detected:</p>

				<?php if ( $this->has_legacy_version( '3' ) ) : ?>
				<div>
					<p>SearchWP 3.x</p>
					<form action="" method="GET">
						<?php wp_nonce_field( 'searchwp-remove-3x', 'searchwp_ldr' ); ?>
						<input type="hidden" name="page" value="searchwp-legacy-data-removal">
						<input type="hidden" name="searchwp-action" value="remove-3x">
						<button class="button" type="submit">Remove Data</button>
					</form>
				</div>
				<?php else : ?>
					<p class="description">There is no legacy SearchWP data detected. You may deactivate and remove this plugin.</p>
				<?php endif; ?>
			</div>
		</div>

		<script>
			jQuery(document).ready(function($){
				$('#searchwp-legacy-data-removal form').submit(function(e){
					return confirm('Are you SURE? This CANNOT be undone!');
				});
			});
		</script>

		<style>
			#searchwp-legacy-data-removal div {
				display: flex;
				align-items: center;
			}

			#searchwp-legacy-data-removal div p {
				font-weight: bold;
				margin-right: 2em;
			}
		</style>
		<?php
	}

	/**
	 * Checks for the submitted legacy version.
	 *
	 * @param mixed $version
	 * @return bool
	 */
	public function has_legacy_version( $version ) {
		$has_legacy = false;

		switch ( $version ) {
			case '3':
				$legacy_settings = get_option( 'searchwp_settings' );
				$has_legacy      = ! empty( $legacy_settings );
		}

		return $has_legacy;
	}

	/**
	 * Removal routine for SearchWP 3.x.
	 *
	 * @return void
	 */
	private function remove_3x() {
		global $wpdb;

		// Remove post meta.
		$wpdb->delete( $wpdb->postmeta, array( 'meta_key' => '_searchwp_last_index' ) );
		$wpdb->delete( $wpdb->postmeta, array( 'meta_key' => '_searchwp_attempts' ) );
		$wpdb->delete( $wpdb->postmeta, array( 'meta_key' => '_searchwp_skip' ) );
		$wpdb->delete( $wpdb->postmeta, array( 'meta_key' => '_searchwp_skip_doc_processing' ) );
		$wpdb->delete( $wpdb->postmeta, array( 'meta_key' => '_searchwp_review' ) );

		$persisted_extra_metadata_keys = $wpdb->get_col( $wpdb->prepare(
			"SELECT meta_key FROM $wpdb->postmeta WHERE meta_key LIKE %s GROUP BY meta_key",
			'_searchwp_extra_metadata%'
		) );

		if ( ! empty( $persisted_extra_metadata_keys ) ) {
			foreach ( $persisted_extra_metadata_keys as $persisted_extra_metadata_key ) {
				$wpdb->delete( $wpdb->postmeta, array( 'meta_key' => $persisted_extra_metadata_key ) );
			}
		}

		// Drop custom database tables.
		foreach ( array( 'cf', 'index', 'log', 'tax', 'terms' ) as $table ){
			$tableName = $wpdb->prefix . 'swp_' . $table;

			if ( $tableName == $wpdb->get_var( "SHOW TABLES LIKE '$tableName'" ) ) {
				$wpdb->query( "DROP TABLE $tableName" );
			}
		}

		delete_option( 'searchwp_settings' );
		delete_option( 'searchwp_settings_backup' );
		delete_option( 'searchwp_indexer' );
		delete_option( 'searchwp_purge_queue' );
		delete_option( 'searchwp_version' );
		delete_option( 'searchwp_progress' );
		delete_option( 'searchwp_license_key' );
		delete_option( 'searchwp_paused' );
		delete_option( 'searchwp_last_activity' );
		delete_option( 'searchwp_busy' );
		delete_option( 'searchwp_doing_delta' );
		delete_option( 'searchwp_utf8mb4' );
		delete_option( 'searchwp_advanced' );
		delete_option( 'searchwp_waiting' );
		delete_option( 'searchwp_delta_attempts' );
		delete_option( 'searchwp_processing_purge_queue' );
		delete_option( 'searchwp_transient' );
		delete_option( 'swppurge_transient' );
	}
}

new SearchWPLegacyDataRemoval();
